package com.ruoyi.system.controller.user;

import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.common.core.utils.poi.ExcelUtil;
import com.ruoyi.common.core.web.controller.BaseController;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.core.web.page.TableDataInfo;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.security.annotation.InnerAuth;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.system.api.domain.SysRole;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.system.api.model.LoginUser;
import com.ruoyi.system.enums.BussLogEnum;
import com.ruoyi.system.service.ISnTemplatePermsService;
import com.ruoyi.system.service.ISysBussLogService;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysPermissionService;
import com.ruoyi.system.service.ISysPostService;
import com.ruoyi.system.service.ISysRoleService;
import com.ruoyi.system.service.ISysUserService;
import java.io.IOException;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

@RestController
@RequestMapping({"/dealer"})
public class SysDealerController extends BaseController {
   @Autowired
   private ISysUserService userService;
   @Autowired
   private ISysRoleService roleService;
   @Autowired
   private ISysDeptService deptService;
   @Autowired
   private ISysPostService postService;
   @Autowired
   private ISysPermissionService permissionService;
   @Autowired
   private ISnTemplatePermsService templatePermsService;
   @Autowired
   private ISysBussLogService sysBussLogService;

   @RequiresPermissions({"system:dealer:list"})
   @GetMapping({"/list"})
   public TableDataInfo list(SysUser user) {
      if (user.getParams().containsKey("isPage")) {
         if ("true".equalsIgnoreCase(user.getParams().get("isPage").toString())) {
            this.startPage();
         }
      } else {
         this.startPage();
      }

      user.setUserType("dealer");
      List<SysUser> list = this.userService.selectUserList(user);
      return this.getDataTable(list);
   }

   @GetMapping({"/selectList"})
   public AjaxResult selectList(SysUser user) {
      user.setUserType("dealer");
      return AjaxResult.success(this.userService.selectList(user));
   }

   @Log(
      title = "经销商管理",
      businessType = BusinessType.EXPORT
   )
   @RequiresPermissions({"system:dealer:export"})
   @PostMapping({"/export"})
   public void export(HttpServletResponse response, SysUser user) {
      List<SysUser> list = this.userService.selectUserList(user);
      ExcelUtil<SysUser> util = new ExcelUtil(SysUser.class);
      util.exportExcel(response, list, "经销商数据");
   }

   @Log(
      title = "经销商管理",
      businessType = BusinessType.IMPORT
   )
   @RequiresPermissions({"system:dealer:import"})
   @PostMapping({"/importData"})
   public AjaxResult importData(MultipartFile file, boolean updateSupport) throws Exception {
      ExcelUtil<SysUser> util = new ExcelUtil(SysUser.class);
      List<SysUser> userList = util.importExcel(file.getInputStream());
      String operName = SecurityUtils.getUsername();
      String message = this.userService.importUser(userList, updateSupport, operName);
      return this.success(message);
   }

   @PostMapping({"/importTemplate"})
   public void importTemplate(HttpServletResponse response) throws IOException {
      ExcelUtil<SysUser> util = new ExcelUtil(SysUser.class);
      util.importTemplateExcel(response, "经销商数据");
   }

   @InnerAuth
   @GetMapping({"/info/{username}"})
   public R<LoginUser> info(@PathVariable("username") String username) {
      SysUser sysUser = this.userService.selectUserByUserName(username);
      if (StringUtils.isNull(sysUser)) {
         return R.fail("经销商名或密码错误");
      } else {
         Set<String> roles = this.permissionService.getRolePermission(sysUser);
         Set<String> permissions = this.permissionService.getMenuPermission(sysUser);
         LoginUser sysUserVo = new LoginUser();
         sysUserVo.setSysUser(sysUser);
         sysUserVo.setRoles(roles);
         sysUserVo.setPermissions(permissions);
         return R.ok(sysUserVo);
      }
   }

   @GetMapping({"getInfo"})
   public AjaxResult getInfo() {
      SysUser user = this.userService.selectUserById(SecurityUtils.getUserId());
      Set<String> roles = this.permissionService.getRolePermission(user);
      Set<String> permissions = this.permissionService.getMenuPermission(user);
      AjaxResult ajax = AjaxResult.success();
      ajax.put("user", user);
      ajax.put("roles", roles);
      ajax.put("permissions", permissions);
      return ajax;
   }

   @RequiresPermissions({"system:dealer:query"})
   @GetMapping({"/", "/{userId}"})
   public AjaxResult getInfo(@PathVariable(value = "userId",required = false) Long userId) {
      AjaxResult ajax = AjaxResult.success();
      List<SysRole> roles = this.roleService.selectRoleAll();
      ajax.put("roles", roles.stream().filter((r) -> {
         return r.getRoleKey().equals("dealer");
      }).collect(Collectors.toList()));
      ajax.put("posts", this.postService.selectPostAll());
      if (StringUtils.isNotNull(userId)) {
         SysUser sysUser = this.userService.selectUserById(userId);
         ajax.put("data", sysUser);
         ajax.put("postIds", this.postService.selectPostListByUserId(userId));
         ajax.put("roleIds", sysUser.getRoles().stream().map(SysRole::getRoleId).collect(Collectors.toList()));
      }

      return ajax;
   }

   @RequiresPermissions({"system:dealer:add"})
   @Log(
      title = "经销商管理",
      businessType = BusinessType.INSERT
   )
   @PostMapping
   public AjaxResult add(@Validated @RequestBody SysUser user) {
      this.deptService.checkDeptDataScope(user.getDeptId());
      this.roleService.checkRoleDataScope(user.getRoleIds());
      if (!this.userService.checkUserNameUnique(user)) {
         return this.error("新增经销商'" + user.getUserName() + "'失败，登录账号已存在");
      } else if (StringUtils.isNotEmpty(user.getPhonenumber()) && !this.userService.checkPhoneUnique(user)) {
         return this.error("新增经销商'" + user.getUserName() + "'失败，手机号码已存在");
      } else if (StringUtils.isNotEmpty(user.getEmail()) && !this.userService.checkEmailUnique(user)) {
         return this.error("新增经销商'" + user.getUserName() + "'失败，邮箱账号已存在");
      } else if (StringUtils.isEmpty(user.getCloakId())) {
         return this.error("cloakId不能为空");
      } else {
         user.setUserType("dealer");
         user.setCreateBy(SecurityUtils.getUserId().toString());
         user.setCreateName(SecurityUtils.getLoginUser().getUsername());
         user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
         int num = this.userService.insertUser(user);
         if (user.getParams().containsKey("templateIds")) {
            try {
               List<Integer> templateIds = (List)user.getParams().get("templateIds");
               this.templatePermsService.updateUserPerms((List)templateIds.stream().map(Integer::longValue).collect(Collectors.toList()), user.getUserId());
            } catch (Exception var4) {
            }
         }

         return this.toAjax(num);
      }
   }

   @RequiresPermissions({"system:dealer:edit"})
   @Log(
      title = "经销商管理",
      businessType = BusinessType.UPDATE
   )
   @PutMapping
   public AjaxResult edit(@Validated @RequestBody SysUser user) {
      if (!this.userService.checkUserNameUnique(user)) {
         return this.error("修改经销商'" + user.getUserName() + "'失败，登录账号已存在");
      } else if (StringUtils.isNotEmpty(user.getPhonenumber()) && !this.userService.checkPhoneUnique(user)) {
         return this.error("修改经销商'" + user.getUserName() + "'失败，手机号码已存在");
      } else if (StringUtils.isNotEmpty(user.getEmail()) && !this.userService.checkEmailUnique(user)) {
         return this.error("修改经销商'" + user.getUserName() + "'失败，邮箱账号已存在");
      } else if (StringUtils.isEmpty(user.getCloakId())) {
         return this.error("cloakId不能为空");
      } else {
         SysUser obj = this.userService.selectUserById(user.getUserId());
         if (!Objects.equals(obj.getCloakId(), user.getCloakId())) {
            this.userService.updateUserCloakId(obj.getUserId(), user.getCloakId());
         }

         if (user.getParams().containsKey("templateIds")) {
            try {
               List<Integer> templateIds = (List)user.getParams().get("templateIds");
               this.templatePermsService.updateUserPerms((List)templateIds.stream().map(Integer::longValue).collect(Collectors.toList()), user.getUserId());
            } catch (Exception var4) {
            }
         }

         if (!Objects.equals(user.getServerLimit(), obj.getServerLimit())) {
            this.sysBussLogService.insertSysBussLog(BussLogEnum.OptServerNum.getType(), obj.getUserId(), obj.getUserName(), "服务数量变更 " + obj.getServerLimit() + " => " + user.getServerLimit());
         }

         user.setUpdateBy(SecurityUtils.getUsername());
         return this.toAjax(this.userService.updateUser(user));
      }
   }

   @RequiresPermissions({"system:dealer:remove"})
   @Log(
      title = "经销商管理",
      businessType = BusinessType.DELETE
   )
   @DeleteMapping({"/{userIds}"})
   public AjaxResult remove(@PathVariable Long[] userIds) {
      return ArrayUtils.contains(userIds, SecurityUtils.getUserId()) ? this.error("当前经销商不能删除") : this.toAjax(this.userService.deleteUserByIds(userIds));
   }

   @RequiresPermissions({"system:dealer:resetPwd"})
   @Log(
      title = "经销商管理",
      businessType = BusinessType.UPDATE
   )
   @PutMapping({"/resetPwd"})
   public AjaxResult resetPwd(@RequestBody SysUser user) {
      user.setUpdateBy(SecurityUtils.getUsername());
      return this.toAjax(this.userService.resetPwd(user));
   }

   @RequiresPermissions({"system:dealer:resetPwd"})
   @Log(
      title = "经销商管理",
      businessType = BusinessType.UPDATE
   )
   @PutMapping({"/changeStatus"})
   public AjaxResult changeStatus(@RequestBody SysUser user) {
      user.setUpdateBy(SecurityUtils.getUsername());
      return this.toAjax(this.userService.updateUserStatus(user));
   }
}
